@@ -10,8 +10,8 @@ from textwrap import dedent
 import gffutils
 from gffutils import feature
 from gffutils import merge_criteria as mc
-from nose.tools import assert_raises
 
+import pytest
 
 def test_issue_79():
     gtf = gffutils.example_filename("keep-order-test.gtf")
@@ -91,8 +91,8 @@ def test_issue_107():
         db.interfeatures(db.features_of_type("gene", order_by=("seqid", "start")))
     )
     assert [str(i) for i in interfeatures] == [
-        "chr1\tgffutils_derived\tinter_gene_gene\t6\t9\t.\t.\t.\tID=a,b;",
-        "chr2\tgffutils_derived\tinter_gene_gene\t51\t54\t.\t-\t.\tID=c,d;",
+        "chr1\tgffutils_derived\tinter_gene_gene\t6\t9\t.\t.\t.\tID=a-b;",
+        "chr2\tgffutils_derived\tinter_gene_gene\t51\t54\t.\t-\t.\tID=c-d;",
     ]
 
 
@@ -324,9 +324,12 @@ def test_issue_157():
     #   TypeError: merge() got an unexpected keyword argument 'ignore_strand'
     #
     # Now changing to ValueError and suggesting a fix. 
-    assert_raises(ValueError, db.children_bp, gene, child_featuretype='exon', merge=True, ignore_strand=True)
-    assert_raises(ValueError, db.children_bp, gene, ignore_strand=True, nonexistent=True)
-    assert_raises(TypeError, db.children_bp, gene, nonexistent=True)
+    with pytest.raises(ValueError):
+        db.children_bp(gene, child_featuretype='exon', merge=True, ignore_strand=True)
+    with pytest.raises(ValueError):
+        db.children_bp(gene, ignore_strand=True, nonexistent=True)
+    with pytest.raises(TypeError):
+        db.children_bp(gene, nonexistent=True)
 
     # The way to do it now is the following (we can omit the mc.feature_type
     # since we're preselecting for exons anyway):
@@ -385,22 +388,6 @@ def test_issue_174():
     assert observed[8] == ['9', '10'] 
     assert observed[9] == ['10', '11']
 
-
-def test_issue_181():
-    db = gffutils.create_db(
-        gffutils.example_filename('issue181.gff'),
-        ':memory:')
-    introns = db.create_introns()
-
-    # This now warns that the provided ID key has multiple values.
-    assert_raises(ValueError, db.update, introns)
-
-    # The fix is to provide a custom intron ID converter.
-    def intron_id(f):
-        return ','.join(f['ID'])
-
-    db.update(introns, id_spec={'intron': [intron_id]})
-
 def test_issue_197():
 
     # Previously this would fail with ValueError due to using the stop position
@@ -410,11 +397,18 @@ def test_issue_197():
     genes = list(db.features_of_type('gene'))
     igss = list( db.interfeatures(genes,new_featuretype='intergenic_space') )
 
+
+    # Prior to PR #219, multiple IDs could be created by interfeatures, which
+    # in turn was patched here by providing the transform to db.update. With
+    # #219, this ends up being a no-op because ID is a single value by the time
+    # it gets to the transform function.
+    #
+    # However, keeping the test as-is to ensure backward-compatibility.
     def transform(f):
         f['ID'] = [ '-'.join(f.attributes['ID']) ]
         return f
 
-    db = db.update(igss, transform=transform, merge_strategy='error')
+    db = db.update(igss, transform=transform,  merge_strategy='error')
 
     obs = list(db.features_of_type('intergenic_space'))
     for i in obs:
@@ -427,3 +421,187 @@ def test_issue_197():
         'tig00000492\tgffutils_derived\tintergenic_space\t50071\t50071\t.\t-\t.\tID=gene4-gene5',
         'tig00000492\tgffutils_derived\tintergenic_space\t50076\t50089\t.\t-\t.\tID=gene5-gene6',
     ]
+
+def test_issue_198():
+    line = 'NC_000001.11	BestRefSeq	gene	14362	29370	.	-	.	gene_id "WASH7P"; transcript_id ""; db_xref "GeneID:653635"; db_xref "HGNC:HGNC:38034"; description "WASP family homolog 7, pseudogene"; gbkey "Gene"; gene "WASH7P"; gene_biotype "transcribed_pseudogene"; gene_synonym "FAM39F"; gene_synonym "WASH5P"; pseudo "true";'
+
+    # Original issue #198 is that this fails with:
+    #
+    #   gffutils.exceptions.AttributeStringError: Internally inconsistent
+    #   attributes formatting: some have repeated keys, some do not.
+    #
+    # This is because the dialect inference sees the two db_xref keys, and
+    # correctly assumes the dialect uses repeated keys rather than
+    # multiple, comma-separated values -- but there's a comma in the
+    # description.
+    #
+    # So we need to figure out the best way of interpreting a comma in cases
+    # like this. It seems like the best solution is to assume that the presence
+    # of repeated keys always wins.
+    f = feature.feature_from_line(line)
+
+    assert f.attributes['description'] == ['WASP family homolog 7, pseudogene']
+
+    # If we remove one of the db_xref keys, then the parser sees the comma and
+    # figures it's a multivalue key.
+    line = 'NC_000001.11	BestRefSeq	gene	14362	29370	.	-	.	gene_id "WASH7P"; transcript_id ""; db_xref "GeneID:653635"; description "WASP family homolog 7, pseudogene"; gbkey "Gene"; gene "WASH7P"; gene_biotype "transcribed_pseudogene"; gene_synonym "FAM39F"; gene_synonym "WASH5P"; pseudo "true";'
+    f = feature.feature_from_line(line)
+
+    # Previous result, note leading space --------------------------->| |
+    # assert f.attributes['description'] == ['WASP family homolog 7', ' pseudogene']
+    assert f.attributes['description'] == ['WASP family homolog 7, pseudogene']
+
+    # But removing that space before "pseudogene" means it's interpreted as
+    # a multivalue attribute
+    line = 'NC_000001.11	BestRefSeq	gene	14362	29370	.	-	.	gene_id "WASH7P"; transcript_id ""; db_xref "GeneID:653635"; description "WASP family homolog 7,pseudogene"; gbkey "Gene"; gene "WASH7P"; gene_biotype "transcribed_pseudogene"; gene_synonym "FAM39F"; gene_synonym "WASH5P"; pseudo "true";'
+    f = feature.feature_from_line(line)
+    assert f.attributes['description'] == ['WASP family homolog 7', 'pseudogene']
+
+    # Confirm behavior of corner cases like a trailing comma
+    line = "chr17	RefSeq	CDS	6806527	6806553	.	+	0	Name=CDS:NC_000083.5:LOC100040603;Parent=XM_001475631.1,"
+    f = feature.feature_from_line(line)
+    assert f.attributes['Parent'] == ['XM_001475631.1', '']
+
+
+def test_issue_207():
+
+    def _check(txt, expected_keys, dialect_trailing_semicolon):
+        db = gffutils.create_db(txt.replace(' ', '\t'), ':memory:', from_string=True)
+        assert [list(f.attributes.keys()) for f in db.all_features()] == expected_keys
+        assert db.dialect['trailing semicolon'] == dialect_trailing_semicolon
+
+    # All lines have trailing semicolon
+    _check(
+        txt=dedent("""\
+        chr1 AUGUSTUS gene 68330 73621 1 - . ID=g1903;
+        chr1 AUGUSTUS mRNA 68330 73621 1 - . ID=g1903.t1;Parent=g1903;
+        chr1 Pfam protein_match 73372 73618 1 - . ID=g1903.t1.d1;Parent=g1903.t1;
+        chr1 Pfam protein_hmm_match 73372 73618 1 - . ID=g1903.t1.d1.1;Parent=g1903.t1.d1;
+        """),
+        expected_keys = [
+            ['ID'],
+            ['ID', 'Parent'],
+            ['ID', 'Parent'],
+            ['ID', 'Parent'],
+        ],
+        dialect_trailing_semicolon=True
+    )
+
+    # First two lines have trailing semicolon. However, the heuristics of
+    # dialect selection, which favor attributes with more values (assuming more
+    # information), decides that this file does NOT have trailing semicolons.
+    _check(
+        txt=dedent("""\
+        chr1 AUGUSTUS gene 68330 73621 1 - . ID=g1903;
+        chr1 AUGUSTUS mRNA 68330 73621 1 - . ID=g1903.t1;Parent=g1903;
+        chr1 Pfam protein_match 73372 73618 1 - . ID=g1903.t1.d1;Parent=g1903.t1
+        chr1 Pfam protein_hmm_match 73372 73618 1 - . ID=g1903.t1.d1.1;Parent=g1903.t1.d1
+        """),
+        expected_keys = [
+            ['ID', ''],
+            ['ID', 'Parent', ''],
+            ['ID', 'Parent'],
+            ['ID', 'Parent'],
+        ],
+        dialect_trailing_semicolon=False,
+    )
+
+    # APPARENTLY INCONSISTENT: The only thing difference here is that the
+    # Parent attribute has been removed, otherwise matches above (first two
+    # have trailing semicolon). But now there are no empty keys.
+    #
+    # This is expected behavior, because there are no attributes with more keys
+    # as above to give higher weight, and to break the tie between with and
+    # without trailing semicolon, falls back to first dialect observed.
+    _check(
+        txt=dedent("""\
+        chr1 AUGUSTUS gene 68330 73621 1 - . ID=g1903;
+        chr1 AUGUSTUS mRNA 68330 73621 1 - . ID=g1903.t1;
+        chr1 Pfam protein_match 73372 73618 1 - . ID=g1903.t1.d1
+        chr1 Pfam protein_hmm_match 73372 73618 1 - . ID=g1903.t1.d1.1
+        """),
+        expected_keys=[
+            ['ID'],
+            ['ID'],
+            ['ID'],
+            ['ID']
+        ],
+        dialect_trailing_semicolon=True,
+    )
+
+    # We can convince the heuristics to think there should be NO trailing
+    # semicolon by giving one more line as evidence. Only difference is from
+    # above is the last line.
+    _check(
+        txt=dedent("""\
+        chr1 AUGUSTUS gene 68330 73621 1 - . ID=g1903;
+        chr1 AUGUSTUS mRNA 68330 73621 1 - . ID=g1903.t1;
+        chr1 Pfam protein_match 73372 73618 1 - . ID=g1903.t1.d1
+        chr1 Pfam protein_hmm_match 73372 73618 1 - . ID=g1903.t1.d1.1
+        chr1 Pfam protein_hmm_match 73372 73618 1 - . ID=g1904.t1.d1.1
+        """),
+        expected_keys=[
+            ['ID', ''],
+            ['ID', ''],
+            ['ID'],
+            ['ID'],
+            ['ID'],
+        ],
+        dialect_trailing_semicolon=False,
+    )
+
+
+    # Again seems inconsistent at first, but heuristics break ties by
+    # preferring first dialect, which here is no trailing semicolon.
+    _check(
+        txt=dedent("""\
+        chr1 AUGUSTUS gene 68330 73621 1 - . ID=g1903
+        chr1 AUGUSTUS mRNA 68330 73621 1 - . ID=g1903.t1
+        chr1 Pfam protein_match 73372 73618 1 - . ID=g1903.t1.d1;
+        chr1 Pfam protein_hmm_match 73372 73618 1 - . ID=g1903.t1.d1.1;
+        """),
+        expected_keys=[
+            ['ID'],
+            ['ID'],
+            ['ID', ''],
+            ['ID', '']
+        ],
+        dialect_trailing_semicolon=False,
+    )
+
+
+def test_issue_213():
+    # GFF header directives seem to be not parsed when building a db from
+    # a file, even though it seems to work fine from a string.
+    data = dedent(
+        """
+    ##gff-version 3
+    .	.	.	.	.	.	.	.
+    .	.	.	.	.	.	.	.
+    .	.	.	.	.	.	.	.
+    .	.	.	.	.	.	.	.
+    """
+    )
+
+    # Ensure directives are parsed from DataIterator
+    it = gffutils.iterators.DataIterator(data, from_string=True)
+    assert it.directives == ["gff-version 3"]
+
+
+    # Ensure they're parsed into the db from a string
+    db = gffutils.create_db(data, dbfn=":memory:", from_string=True, verbose=False)
+    assert db.directives == ["gff-version 3"], db.directives
+
+    # Ensure they're parsed into the db from a file
+    tmp = tempfile.NamedTemporaryFile(delete=False).name
+    with open(tmp, "w") as fout:
+        fout.write(data + "\n")
+    db = gffutils.create_db(tmp, ":memory:")
+    assert db.directives == ["gff-version 3"], db.directives
+    assert len(db.directives) == 1
+
+    # Ensure they're parsed into the db from a file, and going to a file (to
+    # exactly replicate example in #213)
+    db = gffutils.create_db(tmp, dbfn='issue_213.db', force=True)
+    assert db.directives == ["gff-version 3"], db.directives
+    assert len(db.directives) == 1
